/*
 *	The EchoMail Message Traffic Statistics program
 * 
 *	Copyright (c) 1987 - Joe Keenan, Black Cat Software
 *
 *	in Ecosoft C88...
 *
 *	Version history:
 *
 *	0.00	10/17/87	First Try....
 *	0.10	10/31/87	First Release
 */

#include <dir.h>
#include <malloc.h>
#include <math.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <values.h>
#include <FidoNet.h>

typedef	struct Traffic {

	int	DayTotal;		/* messages seen today */

	int	Samples;		/* samples (days) this week */
	int	WeekMin;		/* lowest daily count */
	int	WeekMax;		/* highest daily count */
	int	WeekTotal;		/* sum of daily counts */
	long	WeekSquares;		/* sum of squares of daily counts */

}	Traffic;

typedef	struct	XListRecord {
	char	tag[30];
	struct	XListRecord *next;
}	XListRecord;

char	CopyRight[] = "\nEchoStat %s, by Joseph P. Keenan Jr.  Matrix 1:109/661\n\
Copyright (C) 1987, Black Cat Software\n";
char	Version[] = "0.00";

int	debug = 0;
int 	noUpdate = 0;
int	doCount = 0;
int	DailyRpt = 0;
int	WeeklyRpt = 0;
int	StatRpt = 0;
int	LinkRecord = 0;

XListRecord *XList = NULL;

char	EchoList[132] = "F:\\EchoMail\\EchoStat.Lst";
char	DailyRptFile[132] = "F:\\EchoMail\\EchoStat.Dly";
char	WeeklyRptFile[132] = "F:\\EchoMail\\EchoStat.Wky";
char	StatRptFile[132] = "F:\\EchoMail\\EchoStat.Rpt";
char	sep[] = " \t\r\n";

main(int argc, char *argv[])
{
	int carg;
	XListRecord *xp;

	printf(CopyRight, Version);

	/* read the command line arguments */

	for (carg = 1; carg < argc; carg++)

		if (argv[carg][0] == '-')

			switch(argv[carg][1]) {

			case '0':	LinkRecord = 0;
					break;

			case '1':	LinkRecord = 1;
					break;

			case 'C':
			case 'c':	doCount = 1;
					break;

			case 'D':
			case 'd':	debug = 1;
					break;

			case 'E':
			case 'e':	strcpy(EchoList,argv[++carg]);
					if (debug)
						printf("EchoMail List file is: %s\n",EchoList);
					break;

			case 'N':
			case 'n':	noUpdate = 1;
					break;

			case 'R':
			case 'r':	switch(argv[carg][2]) {

					case 'D':
					case 'd':	DailyRpt = 1;
							if ((argv[carg+1][0] != '-') && (carg < argc - 1))
								strcpy(DailyRptFile, argv[++carg]);
							if (debug)
								printf("Generating Daily Report: %s\n",DailyRptFile);
							break;

					case 'S':
					case 's':	StatRpt = 1;
							if ((argv[carg+1][0] != '-') && (carg < argc - 1))
								strcpy(StatRptFile, argv[++carg]);
							if (debug)
								printf("Generating Status Report: %s\n",StatRptFile);
							break;

					case 'W':
					case 'w':	WeeklyRpt = 1;
							if ((argv[carg+1][0] != '-') && (carg < argc - 1))
								strcpy(WeeklyRptFile, argv[++carg]);
							if (debug)
								printf("Generating Weekly Report: %s\n",WeeklyRptFile);
							break;
					}
					break;
			case 'X':
			case 'x':	if (debug)
						printf("Processing excusion list:");
					for (XList = NULL; carg + 1 < argc; carg++) {
						if (argv[carg + 1][0] == '-')
							break;
						if ((xp = (XListRecord *) malloc(sizeof(XListRecord))) == NULL) {
							printf("error allocating memory for exclusion list\n");
							exit(-1);
						}
						xp->next = XList;
						XList = xp;
						strcpy(xp->tag,argv[carg + 1]);
					}
					if (debug) {
						for (xp = XList; xp != NULL; xp = xp->next)
							printf(" %s", xp->tag);
						printf(" excluded.\n");
					}
					break;

			}

	/* are we counting this time? */

	if (doCount) 
		CountMsgs();

	/* if reports were requested, do them */

	if (StatRpt)
		StatReport();

	if (DailyRpt)
		DayReport();

	if (WeeklyRpt)
		WeekReport();

}

void CountMsgs()
{
	FILE	*listfp;
	char	scratch[256], Path[132], Tag[30];

	/* loop through the echomail areas, counting new messages in each */

	if ((listfp = fopen(EchoList,"r")) == NULL) {
		printf("EchoStat ERROR: Can't open area list file %s, aborting...\n",EchoList);
		exit(1);
	}

	while (fgets(scratch, 256, listfp) != NULL) {

		strcpy(Path,strtok(scratch,sep));

		/* does this look like a area Pathname ? */

		if ((Path[0] == '\\') ||
		    ((Path[1] == ':') && (Path[2] == '\\'))) {

			strcpy(Tag, strtok(NULL,sep));
			CountArea(Path, Tag);

		} else {		/* second one, Opus style ? */

			strcpy(Path,strtok(NULL,sep));

			if ((Path[0] == '\\') ||
			    ((Path[1] == ':') && (Path[2] == '\\'))) {
	
				strcpy(Tag, strtok(NULL,sep));
				CountArea(Path, Tag);
			}
		}
	}

	fclose(listfp);

}

/* 	Try and figure out how many messages have been added to this	*/
/*	since the last time it was scanned.  Add that count to the	*/
/*	Weekly and Daily count files in that area.			*/

void CountArea(char *Path, char *Tag)
{
	char	scratch[132];
	int	LastScan, Count;
	struct	Traffic	Info;
	FILE	*fp;
	DIR	*dp;
	struct	direct	*direntry;
	DiskMsg	Msg;
	
	if (debug)
		printf("Looking at message area %s\n",Path);

	if (Path[strlen(Path)-1] != '\\')
		strcat(Path,"\\");

	sprintf(scratch,"%s1.msg",Path);

	if ((fp = fopen(scratch,"rb")) == NULL) {

		/* no marker message found, assume last scan is 1 */

		LastScan = 1;

	} else {

		/* get the header of the message, and use link field */

		fread(&Msg, sizeof(DiskMsg), 1, fp);

		if (LinkRecord == 0)
			LastScan = Msg.replyTo;
		else
			LastScan = Msg.nextReply;

		fclose(fp);
	}

	if (debug)
		printf("Last scanned message = %d\n",LastScan);

	/* 	Now, go through the directory.  Count up the number of 	*/
	/*	messages that have a higher number than the last scan	*/
	/*	marker.  That's the number of messages added.		*/

	Count = 0;

	sprintf(scratch,"%s*.msg",Path);
	dp = opendir(scratch);

	/* This is the part that takes the longest, but I don't see	*/
	/* any way to speed it up....					*/

	while (1) {
		direntry = readdir(dp);
		if ((direntry == NULL ) || (direntry->d_name[0] == 0))	/* no more entries */
			break;

		if (atoi(direntry->d_name) > LastScan)
			Count++;
	}

	closedir(dp);


	/* if there were new messages, update the data files */

	if (Count > 0) { 

		printf("New messages in %s: %d\n",Tag,Count);

		/* Update the counts */

		sprintf(scratch,"%sMTRAFFIC.DAT",Path);

		if ((fp = fopen(scratch,"r+b")) == NULL) {

			/* no data file, initialize one */

			Info.DayTotal = 0;
			Info.Samples = 0;
			Info.WeekMax = 0;
			Info.WeekMin = MAXINT;
			Info.WeekTotal = 0;
			Info.WeekSquares = 0l;

			/* and open the file anyway */

			fp = fopen(scratch,"wb");

		} else {

			fread(&Info, sizeof(struct Traffic), 1, fp);
			rewind(fp);

		}

		/* Update the traffic info */

		Info.DayTotal  += Count;
		Info.WeekTotal += Count;

		fwrite(&Info, sizeof(struct Traffic), 1, fp);
		fclose(fp);

		if (debug)
			printf("New Messages in %s: %d\n", Path, Count);

	} 
}	

void StatReport()
{
	FILE	*listfp, *Reportfp, *Datafp;
	char	scratch[132], Tag[32], Path[132];
	int	onLine, DaySum, WeekSum;
	struct	Traffic Info;
	time_t	ourtime;
	struct tm *btime;

	ourtime = time(&ourtime);
	btime = localtime(&ourtime);	

	DaySum = WeekSum = 0;
	onLine = 0;

	/* Open the output files and generate header info */

	if ((Reportfp = fopen(StatRptFile,"w")) == NULL) {
		printf("Unable to open Status Report File %s\n",StatRptFile);
		return;
	} else
		fprintf(Reportfp,"\t EchoMail Traffic Status Report, %02d/%02d/%02d  %02d:%02d:%02d\n\n",
			btime->tm_mon+1, btime->tm_mday, btime->tm_year,
			btime->tm_hour, btime->tm_min, btime->tm_sec);

	if ((listfp = fopen(EchoList,"r")) == NULL) {
		printf("EchoStat ERROR: Can't open area list file %s, aborting...\n",EchoList);
		exit(1);
	}

	/* go through the echomail area list, getting the count information */

	while (fgets(scratch, 256, listfp) != NULL) {

		strcpy(Path,strtok(scratch,sep));

		/* does this look like a area Pathname ? */

		if ((Path[0] == '\\') ||
		    ((Path[1] == ':') && (Path[2] == '\\'))) {

			strcpy(Tag, strtok(NULL,sep));

		} else {		/* second one, Opus style ? */

			strcpy(Path,strtok(NULL,sep));

			if ((Path[0] == '\\') ||
			    ((Path[1] == ':') && (Path[2] == '\\'))) {
	
				strcpy(Tag, strtok(NULL,sep));

			} else		/* skip this line */
				continue;
		}

		/* skip this one? */

		if (inXList(Tag)) {
			if (debug)
				printf("Skipping %s\n",Tag);
			continue;
		}

		/* Open the Data file and get info */

		if (Path[strlen(Path)-1] != '\\')
			strcat(Path,"\\");
		sprintf(scratch,"%sMTRAFFIC.DAT",Path);

		if (debug)
			printf("Getting info from %s\n",scratch);

		if ((Datafp = fopen(scratch,"rb")) == NULL)
			continue;
		else {
			fread(&Info, sizeof(struct Traffic), 1, Datafp);
			fclose(Datafp);
		}
	
		if (debug) {
			printf("DayTotal    = %d\n",Info.DayTotal);
			printf("Samples     = %d\n",Info.Samples);
			printf("WeekMin     = %d\n",Info.WeekMin);
			printf("WeekMax     = %d\n",Info.WeekMax);
			printf("WeekTotal   = %d\n",Info.WeekTotal);
			printf("WeekSquares = %ld\n",Info.WeekSquares);
		}

		fprintf(Reportfp,"%-12s %4d/%-4d  ", Tag, Info.DayTotal, Info.WeekTotal);

		if (++onLine == 3) {
			fprintf(Reportfp,"\n");
			onLine = 0;
		}

		DaySum += Info.DayTotal;
		WeekSum += Info.WeekTotal; 
	}
	fclose(listfp);

	fprintf(Reportfp,"\n\n\tTotal Messages, Today: %d  This Week: %d\n",DaySum, WeekSum);
	fclose(Reportfp);

}

void DayReport()
{
	FILE	*listfp, *Reportfp, *Datafp;
	char	scratch[132], Tag[32], Path[132];
	int	onLine, DaySum, WeekSum;
	struct	Traffic Info;
	time_t	ourtime;
	struct tm *btime;

	ourtime = time(&ourtime);
	btime = localtime(&ourtime);	

	DaySum = WeekSum = 0;
	onLine = 0;

	/* Open the output files and generate header info */

	if ((Reportfp = fopen(DailyRptFile,"w")) == NULL) {
		printf("Unable to open Daily Report File %s\n",DailyRptFile);
		return;
	} else
		fprintf(Reportfp,"\t Daily EchoMail Traffic Report, %02d/%02d/%02d  %02d:%02d:%02d\n\n",
			btime->tm_mon+1, btime->tm_mday, btime->tm_year,
			btime->tm_hour, btime->tm_min, btime->tm_sec);

	if ((listfp = fopen(EchoList,"r")) == NULL) {
		printf("EchoStat ERROR: Can't open area list file %s, aborting...\n",EchoList);
		exit(1);
	}

	/* go through the echomail area list, getting the count information */

	while (fgets(scratch, 256, listfp) != NULL) {

		strcpy(Path,strtok(scratch,sep));

		/* does this look like a area Pathname ? */

		if ((Path[0] == '\\') ||
		    ((Path[1] == ':') && (Path[2] == '\\'))) {

			strcpy(Tag, strtok(NULL,sep));

		} else {		/* second one, Opus style ? */

			strcpy(Path,strtok(NULL,sep));

			if ((Path[0] == '\\') ||
			    ((Path[1] == ':') && (Path[2] == '\\'))) {
	
				strcpy(Tag, strtok(NULL,sep));

			} else		/* skip this line */
				continue;
		}

		/* skip this one? */

		if (inXList(Tag)) {
			if (debug)
				printf("Skipping %s\n",Tag);
			continue;
		}

		/* Open the Data file and get info */

		if (Path[strlen(Path)-1] != '\\')
			strcat(Path,"\\");
		sprintf(scratch,"%sMTRAFFIC.DAT",Path);

		if (debug)
			printf("Getting info from %s\n",scratch);

		if ((Datafp = fopen(scratch,"r+b")) == NULL)
			continue;
		else {
			fread(&Info, sizeof(struct Traffic), 1, Datafp);
			rewind(Datafp);
		}
	
		if (debug) {
			printf("DayTotal    = %d\n",Info.DayTotal);
			printf("Samples     = %d\n",Info.Samples);
			printf("WeekMin     = %d\n",Info.WeekMin);
			printf("WeekMax     = %d\n",Info.WeekMax);
			printf("WeekTotal   = %d\n",Info.WeekTotal);
			printf("WeekSquares = %ld\n",Info.WeekSquares);
		}

		/* put it in the report */

		fprintf(Reportfp,"%-14s %4d  ", Tag, Info.DayTotal);

		if (++onLine == 3) {
			fprintf(Reportfp,"\n");
			onLine = 0;
		}

		DaySum += Info.DayTotal;
		WeekSum += Info.WeekTotal;

		/* update the info block */

		Info.Samples++;
		Info.WeekSquares += ((long) Info.DayTotal * (long) Info.DayTotal);

		if (Info.DayTotal < Info.WeekMin)
			Info.WeekMin = Info.DayTotal;

		if (Info.DayTotal > Info.WeekMax)
			Info.WeekMax = Info.DayTotal;

		Info.DayTotal = 0;

		if (! noUpdate)
			fwrite(&Info, sizeof(struct Traffic), 1, Datafp);
		fclose(Datafp);

	}
	fclose(listfp);

	fprintf(Reportfp,"\n\n\tTotal Messages, Today: %d  This Week: %d\n",DaySum, WeekSum);
	fclose(Reportfp);

}

void WeekReport()
{
	FILE	*listfp, *Reportfp, *Datafp;
	char	scratch[132], Tag[32], Path[132];
	int	onLine, WeekSum;
	float	StdDev, Mean;
	struct	Traffic Info;
	time_t	ourtime;
	struct tm *btime;

	ourtime = time(&ourtime);
	btime = localtime(&ourtime);	

	WeekSum = 0;
	onLine = 0;

	/* Open the output files and generate header info */

	if ((Reportfp = fopen(WeeklyRptFile,"w")) == NULL) {
		printf("Unable to open Weekly Report File %s\n",WeeklyRptFile);
		return;
	}

	fprintf(Reportfp,"\t Weekly EchoMail Traffic Report, %02d/%02d/%02d  %02d:%02d:%02d\n\n",
		btime->tm_mon+1, btime->tm_mday, btime->tm_year,
		btime->tm_hour, btime->tm_min, btime->tm_sec);
	fprintf(Reportfp," Conference      Total    Average  Max Msgs  Min Msgs  Standard\n");
	fprintf(Reportfp,"    Name        Messages  per Day  per Day   per Day   Deviation\n");
	fprintf(Reportfp," ----------     --------  -------  --------  --------  ---------\n");

	if ((listfp = fopen(EchoList,"r")) == NULL) {
		printf("EchoStat ERROR: Can't open area list file %s, aborting...\n",EchoList);
		exit(1);
	}

	/* go through the echomail area list, getting the count information */

	while (fgets(scratch, 256, listfp) != NULL) {

		strcpy(Path,strtok(scratch,sep));

		/* does this look like a area Pathname ? */

		if ((Path[0] == '\\') ||
		    ((Path[1] == ':') && (Path[2] == '\\'))) {

			strcpy(Tag, strtok(NULL,sep));

		} else {		/* second one, Opus style ? */

			strcpy(Path,strtok(NULL,sep));

			if ((Path[0] == '\\') ||
			    ((Path[1] == ':') && (Path[2] == '\\'))) {
	
				strcpy(Tag, strtok(NULL,sep));

			} else		/* skip this line */
				continue;
		}

		/* skip this one? */

		if (inXList(Tag)) {
			if (debug)
				printf("Skipping %s\n",Tag);
			continue;
		}

		/* Open the Data file and get info */

		if (Path[strlen(Path)-1] != '\\')
			strcat(Path,"\\");
		sprintf(scratch,"%sMTRAFFIC.DAT",Path);

		if (debug)
			printf("Getting info from %s\n",scratch);

		if ((Datafp = fopen(scratch,"r+b")) == NULL)
			continue;
		else {
			fread(&Info, sizeof(struct Traffic), 1, Datafp);
			rewind(Datafp);
		}
	
		if (debug) {
			printf("DayTotal    = %d\n",Info.DayTotal);
			printf("Samples     = %d\n",Info.Samples);
			printf("WeekMin     = %d\n",Info.WeekMin);
			printf("WeekMax     = %d\n",Info.WeekMax);
			printf("WeekTotal   = %d\n",Info.WeekTotal);
			printf("WeekSquares = %ld\n",Info.WeekSquares);
		}

		/* put it in the report */

		if (Info.Samples > 1) {

			StdDev = sqrt((float) ( ( (Info.Samples * Info.WeekSquares) - (Info.WeekTotal * Info.WeekTotal) ) ) /
					(float) (Info.Samples * (Info.Samples - 1) ) );

			Mean = (float) Info.WeekTotal / (float) Info.Samples;

		} else {
			StdDev = 0.0;
			Mean = (float) Info.WeekTotal;
		}

		fprintf(Reportfp,"%-15s  %5d    %6.1f    %5d      %5d   %#8.3f\n", Tag, 
			Info.WeekTotal, Mean, Info.WeekMax, Info.WeekMin, StdDev);

		WeekSum += Info.WeekTotal;

		/* update the info block */

		Info.DayTotal = 0;
		Info.Samples = 0;
		Info.WeekMax = 0;
		Info.WeekMin = MAXINT;
		Info.WeekTotal = 0;
		Info.WeekSquares = 0l;

		if (! noUpdate)
			fwrite(&Info, sizeof(struct Traffic), 1, Datafp);
		fclose(Datafp);

	}
	fclose(listfp);

	fprintf(Reportfp,"\n\n\tTotal Messages for the Week: %d\n", WeekSum);
	fclose(Reportfp);

}

int inXList(char *Echo)
{
	XListRecord *xp;

	if (debug)
		printf("looking for %s in exclusion list - ",Echo);

	for (xp = XList; xp != NULL; xp = xp->next) {
		if (strcmp(xp->tag,Echo) == 0) {
			if (debug)
				printf("found\n");
			return(1);
		}
	}

	if (debug)
		printf("not found\n");
	return(0);
}

